/*
 * led_light.h
 *
 * Created: 29.05.2018 11:37:09
 * Author: mathiasc / "Der Andere"
 * Based on the work from senseHaJo (https://github.com/ehajo/SenseHaJo)
 * 
 */ 


#ifndef LED_LIGHT_H_
#define LED_LIGHT_H_

/* Some modes are currently not supported */
typedef enum {
    LED_BLINK_10Hz,
    LED_BLINK_5Hz,
    LED_BLINK_2Hz,
    LED_BLINK_1Hz,
    LED_FADE_IN,  /* IF LED IS OFF THIS WILL DO A FADEOUT */
    LED_FASE_OUT, /* IF LED IS ON THIS WILL DO A FADEOUT */
    LED_FADE_IN_OUT, /* FADEIN / FADEOUT Effect */
    LED_BLINK_OFF,
    LED_STATIC_OFF,
    LED_STATIC_ON,
} LED_BLINK_Frq_t;

typedef enum {
    LED_RED,
    LED_GREEN,
} LED_t;


/**************************************************************************************************
 *    Function      : LED_LIGHT_Init
 *    Description   : Init for the service
 *    Input         : none 
 *    Output        : none
 *....Remarks       : gently borrowed for the Coffemaker MK I / senseeHaJo 
 **************************************************************************************************/
void LED_LIGHT_Init( void );

/**************************************************************************************************
 *    Function      : LED_LIGHT_Task
 *    Description   : Task for the Led's
 *    Input         : none 
 *    Output        : none
 *....Remarks       : needs to be called as often as possible 
 **************************************************************************************************/
void LED_LIGHT_Task( void );


/**************************************************************************************************
 *    Function      : LED_LIGHT_SetLed
 *    Description   : Set the Mode of the Led
 *    Input         : LED, Frequency 
 *    Output        : none
 *....Remarks       : none 
 **************************************************************************************************/
void LED_LIGHT_SetLed( LED_t Led , LED_BLINK_Frq_t Freq );

/**************************************************************************************************
 *    Function      : LED_LIGHT_getLed
 *    Description   : Gets the Mode of the Led
 *    Input         : LED
 *    Output        : Frequency
 *....Remarks       : none 
 **************************************************************************************************/
LED_BLINK_Frq_t LED_LIGHT_getLed( LED_t Led);


#endif /* LED_LIGHT_H_ */